#! /usr/bin/env python
"""
*Created on: ?
*Modified by: Dorjee Gyaltsen (07.01.2016)
*brief: This script will set appropriate directory settings because some methods -
    NetMHCpan, PickPocket, NetMHCcons need this.
"""
from __future__ import print_function

import os
import re
from subprocess import call

class Configure(object):

    def __init__(self, version='20130222'):
        self.version = version
        self.project_dir = os.path.abspath('.')        
        self.src = os.path.join(self.project_dir,'src')
        self.method = os.path.join(self.project_dir,'method')
        
    def generate_setupinfo(self):
        print("Generating setupinfo.py")
        tf = open(os.path.join(self.src, 'setupinfo.template'), 'r')
        content_template = tf.read()
        content = content_template % (self.project_dir)
        tf.close()
        
        of = open(os.path.join(self.src, 'setupinfo.py'), 'w')
        of.write(content)
        of.close()

    def netmhc_4_0(self):
        executable_name = 'netmhc_4_0_executable'
        print(executable_name)
        
        NETMHC_PATH = os.path.join(self.method, executable_name)
        # extract data archive
        self.extract_data_archive(NETMHC_PATH)
        
    def netmhc_3_4(self):
        executable_name = 'netMHC-3.4'
        print(executable_name)
        
        NETMHC_PATH = os.path.join(self.method, executable_name)
        SCRATCH_PATH = os.path.join(NETMHC_PATH,'scratch')
        
        # extract data archive
        self.extract_data_archive(NETMHC_PATH)

        if (os.path.exists(SCRATCH_PATH) == False):
            os.mkdir(SCRATCH_PATH)
        os.chmod(SCRATCH_PATH, 0777) # So the all can write in this directory.

        fname = os.path.join(NETMHC_PATH, 'netMHC.template')
        fname_out = os.path.join(NETMHC_PATH, 'netMHC')

        tf = open(fname, 'r')
        content_template = tf.read()
        content = re.sub('prediction_directory', NETMHC_PATH, content_template)
        tf.close()
        
        print("* netmhc/ann script: {}".format(fname_out))
        
        of = open(fname_out, 'w')
        of.write(content)
        of.close()

        os.chmod(fname_out, 0755)
        
    def netmhcpan_2_8(self):
        executable_name = 'netMHCpan-2.8'
        print(executable_name)
        
        NETMHCPAN_PATH = os.path.join(self.method, executable_name)
        SCRATCH_PATH = os.path.join(NETMHCPAN_PATH,'scratch')
        
        # extract data archive
        self.extract_data_archive(NETMHCPAN_PATH)

        if (os.path.exists(SCRATCH_PATH) == False):
            os.mkdir(SCRATCH_PATH)
        os.chmod(SCRATCH_PATH, 0777) # So the all can write in this directory.

        fname = os.path.join(NETMHCPAN_PATH, 'netMHCpan.template')
        fname_out = os.path.join(NETMHCPAN_PATH, 'netMHCpan')

        tf = open(fname, 'r')
        content_template = tf.read()
        content = content_template % (NETMHCPAN_PATH, SCRATCH_PATH)
        tf.close()

        print('* netmhcpan script:', fname_out)
        of = open(fname_out, 'w')
        of.write(content)
        of.close()

        os.chmod(fname_out, 0755)
        
    def netmhcpan_3_0(self):
        executable_name = 'netmhcpan_3_0_executable'
        print(executable_name)
        
        NETMHCPAN_PATH = os.path.join(self.method, executable_name)
        SCRATCH_PATH = os.path.join(NETMHCPAN_PATH,'scratch')
        
        # extract data archive
        self.extract_data_archive(NETMHCPAN_PATH)
        
    def pickpocket(self):
        executable_name = 'pickpocket-1.1'
        print(executable_name)
        
        PICKPOCKET_PATH = os.path.join(self.method, executable_name)
        SCRATCH_PATH = os.path.join(PICKPOCKET_PATH,'scratch')

        # extract data archive
        self.extract_data_archive(PICKPOCKET_PATH)
        
        if (os.path.exists(SCRATCH_PATH) == False):
            os.mkdir(SCRATCH_PATH)
        os.chmod(SCRATCH_PATH, 0777) # So the all can write in this directory.

        fname = os.path.join(PICKPOCKET_PATH, 'PickPocket.template')
        fname_out = os.path.join(PICKPOCKET_PATH, 'PickPocket')

        tf = open(fname, 'r')
        content_template = tf.read()
        content = content_template % (PICKPOCKET_PATH, SCRATCH_PATH)
        tf.close()
        
        print("* pickpocket script: {}".format(fname_out))
        
        of = open(fname_out, 'w')
        of.write(content)
        of.close()

        os.chmod(fname_out, 0755)
    
    def netmhccons(self):
        executable_name = 'netMHCcons-1.1'
        print(executable_name)
        
        NETMHCCONS_PATH = os.path.join(self.method, executable_name)
        NETMHCPAN_PATH = os.path.join(self.method,'netMHCpan-2.8/netMHCpan')
        NETMHC_PATH = os.path.join(self.method,'netMHC-3.4/netMHC')
        PICKPOCKET_PATH = os.path.join(self.method,'pickpocket-1.1/PickPocket')
        SCRATCH_PATH = os.path.join(NETMHCCONS_PATH,'scratch')
        
        # extract data archive
        self.extract_data_archive(NETMHCCONS_PATH)

        if (os.path.exists(SCRATCH_PATH) == False):
            os.mkdir(SCRATCH_PATH)
        os.chmod(SCRATCH_PATH, 0777) # So the all can write in this directory.

        fname = os.path.join(NETMHCCONS_PATH, 'netMHCcons.template')
        fname_out = os.path.join(NETMHCCONS_PATH, 'netMHCcons')

        tf = open(fname, 'r')
        content_template = tf.read()
        content = content_template % (NETMHCCONS_PATH, NETMHCPAN_PATH, NETMHC_PATH, PICKPOCKET_PATH, SCRATCH_PATH)
        tf.close()
        
        print("* netmhccons script: {}".format(fname_out))

        of = open(fname_out, 'w')
        of.write(content)
        of.close()

        os.chmod(fname_out, 0755)
        
    def extract_data_archive(self, method_path):
        import tarfile
        import sys
        import time
        
        data = 'data.tar.gz'
        print("* extracting {}...".format(data), end="")
        sys.stdout.flush()
        time.sleep(1)
        
        data_path = os.path.join(method_path, data)
        
        if os.path.exists(data_path):
            f = tarfile.open(data_path, 'r:gz')
            try:
                f.extractall(method_path)
            except KeyError:
                print('ERROR: Did not find {} in tar archive'.format(data))
        print("done.")

if __name__ == '__main__':
    config = Configure()
    config.generate_setupinfo()
    config.netmhc_4_0()
    config.netmhc_3_4()
    config.netmhcpan_2_8()
    config.netmhcpan_3_0()
    config.pickpocket()
    config.netmhccons()
    print("All set!")

